/*
 * SPDX-FileCopyrightText: 2024-present Setup Tooling Project
 *
 * SPDX-License-Identifier: Apache-2.0
 */

// Copyright (C) 2024-present Setup Tooling Project

#ifndef LIBSTP_C_H
#define LIBSTP_C_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h> // For bool type
#include <stdarg.h> // For va_list
#include <sys/stat.h>
#include <dirent.h>
#include <pwd.h> // For getpwnam and struct passwd
#include <ncurses.h>

#define PROGRAMS_PATH "/usr/bin/"

#define MAX_SCRIPTS 128
#define MAX_PATH PATH_MAX
#define MAX_NAME NAME_MAX

#if defined(__GNUC__) || defined(__clang__)
    #define DEPRECATED __attribute__((deprecated("This function is deprecated and will be remove in a future release.")))
#else
    #define DEPRECATED
#endif

typedef struct {
    char name[MAX_NAME];
    char path[MAX_PATH];
    mode_t permissions;  // Store file permissions
    time_t modified;
} Script;

typedef struct {
    char *name;
    void (*function)(void);
} MenuItem;

typedef enum {
    PKG_INSTALL,
    PKG_UNINSTALL,
    PKG_SEARCH,
    PKG_UPDATE,
    SYS_UPGRADE
} PackageOperation;

typedef struct {
    const char* name;
    const char* install_cmd;
    const char* uninstall_cmd;
    const char* search_cmd;
    const char* update_cmd;
    const char* upgrade_cmd;
    bool needs_sudo;
} PackageManager;

enum ScriptError {
    SCRIPT_SUCCESS = 0,
    SCRIPT_DIR_ERROR = -1,
    SCRIPT_PERMISSION_ERROR = -2,
    SCRIPT_PATH_ERROR = -3,
    SCRIPT_BOUNDS_ERROR = -4,
    SCRIPT_EXECUTION_ERROR = -5,
    CLOUD_CONFIG_ERROR = -8,
    VM_DETECTION_ERROR = -9
};

// Function declarations
char *strlwr(char *str);
int runCommand(const char* format, ...);
int runCommandWithPassword(const char* password, const char* format, ...);
bool commandExists(const char* command);
int getMenuOption(int maxOption);
void extractOSVersion(char *version);
void extractLinuxDistro(char *distro);
bool isValidPackageName(const char *packageName);
int checkProgramExists(const char* programName);
bool fetchConfig(const char* config_var);
void saveConfigValue(const char* config_var, char response);
char* getSecurePasswordSTP();  // Returns mlocked password - must free with secureWipeAndFreePassword()
void secureWipeString(char* str);  // Securely wipe string in place
void secureWipeAndFreePassword(char* password);  // Securely wipe, unlock, and free password
char* getVersion(const char* programName);
char* set_log_path();
const char* getScriptShell(const char* path, const char* extension);
int compile_and_execute_c(const char* source_path);
int listScripts(const char* directory, Script scripts[], int current_count, int max_scripts);
void displayScriptMenu(Script scripts[], int scriptCount);
int executeScript(const Script* script);
void setConfigFile(const char* path);
const char* getConfigFile(void);
bool hasConfigFile(void);
void display_special_day();
int create_dir(const char* directory);
int scan_arch(void);
void program_version(const char *programsStr);
char* get_real_user(void);
const char* get_arch_string(void);
const char* get_ostree_variant(void);
int download_file(const char* dload_file_path, const char* download_url);
void init_ncurses();
void cleanup_ncurses();
void ndraw_menu(WINDOW *win, MenuItem *items, int selected);
void nshow_message(const char *msg, bool is_error);
bool packageOperation(const char* managerName, PackageOperation op, const char* packageName,    ...);

#ifdef __cplusplus
}
#endif

#endif
